


#include <stdio.h>
#include "ad++.h"

const int P = 4 ;

const int numRepeats1 = 10000 ;

const int numNs = 9 ;
int Ns      [numNs] = {  32,   64,  128,  256,  512, 768, 1024, 1280, 1536} ;
int repeats [numNs] = {1000, 1000, 1000, 1000, 1000, 100,   10,   10,   10} ;
int copies  [numNs] = { 100,  100,  100,   10,   10,   5,    5,    3,    2} ;

// Repeats to amortize timing overheads and make sure everyone is in step.
// Work on multiple copies of arrays to thwart caching.

int main(int argc, char* argv[]) {
  AdlibInit(argc, argv) ;

  Procs1 p(P) ;

  gprintf("Timings for MPI-based remap operations\n") ;

  ON(p) {
    for (int l = 0 ; l < numNs ; l++) {
      const int N = Ns [l] ;
      const int C = copies [l] ;
      const int numRepeats = repeats [l] ;

      int k ;

      BlockRange x(N, p.dim(0)) ;
      CollapsedRange y(N) ;

      Index i(x), j(y) ;

      double t0, t1 ;

      Array2<float>** as = new Array2<float>* [C] ;
      Array2<float>** bs = new Array2<float>* [C] ;
      for(k = 0 ; k < C ; k++) {
        as [k] = new Array2<float>(x, y) ;
        bs [k] = new Array2<float>(y, x) ;

        OVERALL(i) {
	  OVERALL(j) {
	    (*as [k])(i, j) = N * (1 + i) + 1 + j ;
	  } ALLOVER(j) ;
        } ALLOVER(i) ;
      }

      gprintf("\nN = %d\n", N) ;

      // timing loops...

      // Timing schedule construction:

      t0 = MPI_Wtime() ;

      for(k = 0 ; k < numRepeats1 ; k++) {
        const int c = k % C ;
	Remap schedule(bs [c], as [c], sizeof(float)) ;
      }

      t1 = MPI_Wtime() ;

      gprintf("Time per Remap schedule constructor in microseconds: %f\n",
	      1000000 * (t1 - t0) / numRepeats1) ;

      // Timing schedule execution:

      Remap** schedules = new Remap* [C] ;
      for(k = 0 ; k < C ; k++)
        schedules [k] = new Remap(bs [k], as [k], sizeof(float)) ;

      // Full schedule execution...

      t0 = MPI_Wtime() ;

      for(k = 0 ; k < numRepeats ; k++) {
        const int c = k % C ;
	schedules [c]->execute(bs [c]->dat(), as [c]->dat()) ;
      }

      t1 = MPI_Wtime() ;

      for(k = 0 ; k < C ; k++) {
	OVERALL(i) {
	  OVERALL(j) {
	    if((*bs [k])(j, i) != (float) (N * (1 + j) + 1 + i)) {
	      AdlibPrintf("An error was detected in copy %d at (%d, %d)\n",
                          k, (int) j, (int) i) ;
	      AdlibError("aborting") ;
	    }
	  } ALLOVER(j) ;
	} ALLOVER(i) ;
      }

      gprintf("Time per schedule execution in microseconds: %f\n",
	      1000000 * (t1 - t0) / numRepeats) ;

      // Clean up.

      for(k = 0 ; k < C ; k++)
        delete schedules [k] ;
      delete [] schedules ;

      for(k = 0 ; k < C ; k++) {
        delete bs [k] ;
        delete as [k] ;
      }
      delete [] bs ;
      delete [] as ;
    }      

  } NO(p) ;

  AdlibFinalize() ;
}

